@@ -1,26 +1,19 @@
 //
-// "$Id: Fluid_Image.cxx,v 1.7.2.9.2.13 2002/10/03 15:23:46 easysw Exp $"
+// "$Id: Fluid_Image.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Pixmap label support for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2002 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+//     http://www.fltk.org/COPYING.php
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+// Please report all bugs and problems on the following page:
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
@@ -37,16 +30,17 @@ extern void goto_source_dir(); // in fluid.C
 extern void leave_source_dir(); // in fluid.C
 
 void Fluid_Image::image(Fl_Widget *o) {
-  o->image(img);
+  if (o->window() != o) o->image(img);
 }
 
 void Fluid_Image::deimage(Fl_Widget *o) {
-  o->deimage(img);
+  if (o->window() != o) o->deimage(img);
 }
 
 static int pixmap_header_written = 0;
 static int bitmap_header_written = 0;
 static int image_header_written = 0;
+static int jpeg_header_written = 0;
 
 void Fluid_Image::write_static() {
   if (!img) return;
@@ -92,12 +86,43 @@ void Fluid_Image::write_static() {
     }
     write_c("static unsigned char %s[] =\n",
 	    unique_id(this, "idata", fl_filename_name(name()), 0));
-    write_cstring(img->data()[0], ((img->w() + 7) / 8) * img->h());
+    write_cdata(img->data()[0], ((img->w() + 7) / 8) * img->h());
     write_c(";\n");
     write_c("static Fl_Bitmap %s(%s, %d, %d);\n",
 	    unique_id(this, "image", fl_filename_name(name()), 0),
 	    unique_id(this, "idata", fl_filename_name(name()), 0),
 	    img->w(), img->h());
+  } else if (strcmp(fl_filename_ext(name()), ".jpg")==0) {
+    // Write jpeg image data...
+    write_c("\n");
+    if (jpeg_header_written != write_number) {
+      write_c("#include <FL/Fl_JPEG_Image.H>\n");
+      jpeg_header_written = write_number;
+    }
+    write_c("static unsigned char %s[] =\n",
+	    unique_id(this, "idata", fl_filename_name(name()), 0));
+        
+    FILE *f = fl_fopen(name(), "rb");
+    if (!f) {
+      // message = "Can't include binary file. Can't open";
+    } else {
+      fseek(f, 0, SEEK_END);
+      size_t nData = ftell(f);
+      fseek(f, 0, SEEK_SET);
+      if (nData) {
+        char *data = (char*)calloc(nData, 1);
+        if (fread(data, nData, 1, f)==0) { /* ignore */ }
+        write_cdata(data, nData);
+        free(data);
+      }
+      fclose(f);
+    }
+    
+    write_c(";\n");
+    write_c("static Fl_JPEG_Image %s(\"%s\", %s);\n",
+	    unique_id(this, "image", fl_filename_name(name()), 0),
+	    fl_filename_name(name()),
+	    unique_id(this, "idata", fl_filename_name(name()), 0));
   } else {
     // Write image data...
     write_c("\n");
@@ -107,7 +132,7 @@ void Fluid_Image::write_static() {
     }
     write_c("static unsigned char %s[] =\n",
 	    unique_id(this, "idata", fl_filename_name(name()), 0));
-    write_cstring(img->data()[0], (img->w() * img->d() + img->ld()) * img->h());
+    write_cdata(img->data()[0], (img->w() * img->d() + img->ld()) * img->h());
     write_c(";\n");
     write_c("static Fl_RGB_Image %s(%s, %d, %d, %d, %d);\n",
 	    unique_id(this, "image", fl_filename_name(name()), 0),
@@ -116,9 +141,9 @@ void Fluid_Image::write_static() {
   }
 }
 
-void Fluid_Image::write_code(int inactive) {
+void Fluid_Image::write_code(const char *var, int inactive) {
   if (!img) return;
-  write_c("%so->%s(%s);\n", indent(), inactive ? "deimage" : "image",
+  write_c("%s%s->%s(%s);\n", indent(), var, inactive ? "deimage" : "image",
 	  unique_id(this, "image", fl_filename_name(name()), 0));
 }
 
@@ -146,7 +171,7 @@ Fluid_Image* Fluid_Image::find(const char *iname) {
   // no, so now see if the file exists:
 
   goto_source_dir();
-  FILE *f = fopen(iname,"rb");
+  FILE *f = fl_fopen(iname,"rb");
   if (!f) {
     read_error("%s : %s",iname,strerror(errno));
     leave_source_dir();
@@ -212,7 +237,9 @@ Fluid_Image::~Fluid_Image() {
 const char *ui_find_image_name;
 Fluid_Image *ui_find_image(const char *oldname) {
   goto_source_dir();
-  const char *name = fl_file_chooser("Image?","Image Files (*.{bm,bmp,gif,jpg,pbm,pgm,png,ppm,xbm,xpm})",oldname);
+  fl_file_chooser_ok_label("Use Image");
+  const char *name = fl_file_chooser("Image?","Image Files (*.{bm,bmp,gif,jpg,pbm,pgm,png,ppm,xbm,xpm})",oldname,1);
+  fl_file_chooser_ok_label(NULL);
   ui_find_image_name = name;
   Fluid_Image *ret = (name && *name) ? Fluid_Image::find(name) : 0;
   leave_source_dir();
@@ -221,5 +248,5 @@ Fluid_Image *ui_find_image(const char *oldname) {
 
 
 //
-// End of "$Id: Fluid_Image.cxx,v 1.7.2.9.2.13 2002/10/03 15:23:46 easysw Exp $".
+// End of "$Id: Fluid_Image.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
