@@ -1,24 +1,15 @@
 //
-// "$Id: Fl_Group.cxx 8184 2011-01-04 18:28:01Z matt $"
+// "$Id: Fl_Group.cxx 9637 2012-07-24 04:37:22Z matt $"
 //
 // Group widget for the Fast Light Tool Kit (FLTK).
 //
 // Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
-//
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+//     http://www.fltk.org/COPYING.php
 //
 // Please report all bugs and problems on the following page:
 //
@@ -37,6 +28,10 @@
 #include <FL/fl_draw.H>
 #include <stdlib.h>
 
+#include <FL/Fl_Input_Choice.H>
+#include <FL/Fl_Spinner.H>
+
+
 Fl_Group* Fl_Group::current_;
 
 // Hack: A single child is stored in the pointer to the array, while
@@ -832,6 +827,53 @@ void Fl_Group::draw_outside_label(const Fl_Widget& widget) const {
   widget.draw_label(X,Y,W,H,(Fl_Align)a);
 }
 
+
+Fl_Input_Choice::Fl_Input_Choice (int X,int Y,int W,int H,const char*L) 
+: Fl_Group(X,Y,W,H,L) 
+{
+  Fl_Group::box(FL_DOWN_BOX);
+  align(FL_ALIGN_LEFT);				// default like Fl_Input
+  inp_ = new Fl_Input(inp_x(), inp_y(),
+                      inp_w(), inp_h());
+  inp_->callback(inp_cb, (void*)this);
+  inp_->box(FL_FLAT_BOX);		// cosmetic
+  inp_->when(FL_WHEN_CHANGED|FL_WHEN_NOT_CHANGED);
+  menu_ = new InputMenuButton(menu_x(), menu_y(),
+                              menu_w(), menu_h());
+  menu_->callback(menu_cb, (void*)this);
+  menu_->box(FL_FLAT_BOX);				// cosmetic
+  end();
+}
+
+
+Fl_Spinner::Fl_Spinner(int X, int Y, int W, int H, const char *L)
+: Fl_Group(X, Y, W, H, L),
+  input_(X, Y, W - H / 2 - 2, H),
+  up_button_(X + W - H / 2 - 2, Y, H / 2 + 2, H / 2, "@-42<"),
+  down_button_(X + W - H / 2 - 2, Y + H - H / 2,
+               H / 2 + 2, H / 2, "@-42>") 
+{
+  end();
+  
+  value_   = 1.0;
+  minimum_ = 1.0;
+  maximum_ = 100.0;
+  step_    = 1.0;
+  format_  = "%g";
+  
+  align(FL_ALIGN_LEFT);
+  
+  input_.value("1");
+  input_.type(FL_INT_INPUT);
+  input_.when(FL_WHEN_ENTER_KEY | FL_WHEN_RELEASE);
+  input_.callback((Fl_Callback *)sb_cb, this);
+  
+  up_button_.callback((Fl_Callback *)sb_cb, this);
+  
+  down_button_.callback((Fl_Callback *)sb_cb, this);
+}
+
+
 //
-// End of "$Id: Fl_Group.cxx 8184 2011-01-04 18:28:01Z matt $".
+// End of "$Id: Fl_Group.cxx 9637 2012-07-24 04:37:22Z matt $".
 //
