@@ -1,31 +1,25 @@
 //
-// "$Id: fl_dnd_x.cxx,v 1.5.2.7 2003/05/04 21:45:46 easysw Exp $"
+// "$Id: fl_dnd_x.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $"
 //
 // Drag & Drop code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2003 by Bill Spitzak and others.
+// Copyright 1998-2010 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+//     http://www.fltk.org/COPYING.php
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+// Please report all bugs and problems on the following page:
 //
-// Please report all bugs and problems to "fltk-bugs@fltk.org".
+//     http://www.fltk.org/str.php
 //
 
 #include <FL/Fl.H>
 #include <FL/Fl_Window.H>
 #include <FL/x.H>
+#include "flstring.h"
 
 
 extern Atom fl_XdndAware;
@@ -39,8 +33,11 @@ extern Atom fl_XdndStatus;
 extern Atom fl_XdndActionCopy;
 extern Atom fl_XdndFinished;
 //extern Atom fl_XdndProxy;
+extern Atom fl_XdndURIList;
+extern Atom fl_XaUtf8String;
 
 extern char fl_i_own_selection[2];
+extern char *fl_selection_buffer[2];
 
 extern void fl_sendClientMessage(Window window, Atom message,
                                  unsigned long d0,
@@ -82,7 +79,7 @@ static int local_handle(int event, Fl_Window* window) {
 
 int Fl::dnd() {
   Fl_Window *source_fl_win = Fl::first_window();
-  Fl::first_window()->cursor((Fl_Cursor)21);
+  Fl::first_window()->cursor(FL_CURSOR_MOVE);
   Window source_window = fl_xid(Fl::first_window());
   fl_local_grab = grabfunc;
   Window target_window = 0;
@@ -120,8 +117,32 @@ int Fl::dnd() {
       if (local_window) {
 	local_handle(FL_DND_ENTER, local_window);
       } else if (dndversion) {
-	fl_sendClientMessage(target_window, fl_XdndEnter, source_window,
-			     dndversion<<24, XA_STRING, 0, 0);
+        // Send an X-DND message to the target window.  In order to
+	// support dragging of files/URLs as well as arbitrary text,
+	// we look at the selection buffer - if the buffer starts
+	// with a common URI scheme, does not contain spaces, and
+	// contains at least one CR LF, then we flag the data as
+	// both a URI list (MIME media type "text/uri-list") and
+	// plain text.  Otherwise, we just say it is plain text.
+        if ((!strncmp(fl_selection_buffer[0], "file:///", 8) ||
+	     !strncmp(fl_selection_buffer[0], "ftp://", 6) ||
+	     !strncmp(fl_selection_buffer[0], "http://", 7) ||
+	     !strncmp(fl_selection_buffer[0], "https://", 8) ||
+	     !strncmp(fl_selection_buffer[0], "ipp://", 6) ||
+	     !strncmp(fl_selection_buffer[0], "ldap:", 5) ||
+	     !strncmp(fl_selection_buffer[0], "mailto:", 7) ||
+	     !strncmp(fl_selection_buffer[0], "news:", 5) ||
+	     !strncmp(fl_selection_buffer[0], "smb://", 6)) &&
+	    !strchr(fl_selection_buffer[0], ' ') &&
+	    strstr(fl_selection_buffer[0], "\r\n")) {
+	  // Send file/URI list...
+	  fl_sendClientMessage(target_window, fl_XdndEnter, source_window,
+			       dndversion<<24, fl_XdndURIList, XA_STRING, 0);
+        } else {
+	  // Send plain text...
+	  fl_sendClientMessage(target_window, fl_XdndEnter, source_window,
+			       dndversion<<24, fl_XaUtf8String, 0, 0);
+	}
       }
     }
     if (local_window) {
@@ -168,5 +189,5 @@ int Fl::dnd() {
 
 
 //
-// End of "$Id: fl_dnd_x.cxx,v 1.5.2.7 2003/05/04 21:45:46 easysw Exp $".
+// End of "$Id: fl_dnd_x.cxx 8864 2011-07-19 04:49:30Z greg.ercolano $".
 //
